import convert from 'color-convert';

var Util = {
    applyAlpha(base, overlay) {
        const result = [...base];
        result.forEach((channel, index) => {
            result[index] = overlay[index] * overlay[3] + channel * (1 - overlay[3]);
        });
        return result;
    },
    checkLuminance(color, desiredLuminance, threshold = 1) {
        const lchColor = convert.rgb.lch.raw(color);
        const upperBound = desiredLuminance + threshold;
        const lowerBound = desiredLuminance - threshold;
        return lowerBound < lchColor[0] && lchColor[0] < upperBound;
    }
};

class Color {
    constructor(format, value) {
        this.value = value;
        this.format = format;
    }
}

class ConsistentShading {
    constructor(base, shades) {
        this._base = convert[base.format]['lch'].raw(base.value);
        this._shades = new Array();
        this._deltas = new Array();
        shades.forEach((shade, index) => {
            const converted = shade.format === 'lch' ? [...shade.value] : convert[shade.format]['lch'].raw(shade.value);
            this._shades.push(converted);
            this._deltas.push(converted[0] - this._base[0]);
        });
    }
    generate(base, exportFormat = base.format) {
        let exportedColors = [];
        const lchBase = base.format === 'lch' ? [...base.value] : convert[base.format]['lch'].raw(base.value);
        this._deltas.forEach((delta) => {
            let lchGenerated = [...lchBase];
            lchGenerated[0] += delta;
            lchGenerated[0] = Math.min(100, Math.max(0, lchGenerated[0]));
            let convertedLchGenerated = exportFormat === 'lch' ? [...lchGenerated] : convert['lch'][exportFormat](lchGenerated);
            const clgRounded = convertedLchGenerated.map?.((component) => Math.round(component));
            if (typeof clgRounded !== 'undefined') {
                convertedLchGenerated = clgRounded;
            }
            const generatedColor = new Color(exportFormat, convertedLchGenerated);
            exportedColors.push(generatedColor);
        });
        return exportedColors;
    }
    generateAlpha(base, threshold = 1) {
        const lchBase = base.format === 'lch' ? [...base.value] : convert[base.format]['lch'].raw(base.value);
        const rgbBase = base.format === 'rgb' ? [...base.value] : convert[base.format]['rgb'].raw(base.value);
        const lchResults = this.generate(base, 'lch').map((color) => color.value);
        const rgbResults = lchResults.map((color) => convert['lch']['rgb'].raw(color));
        const exportedColors = [];
        rgbResults.forEach((result, index) => {
            const desiredLuminance = lchResults[index][0];
            let currentColor;
            let bottom = 0;
            let top = 1;
            let currentOverlay = [255, 255, 255, 0];
            let overlayIsBlack = 0;
            if (lchResults[index][0] < lchBase[0]) {
                currentOverlay = [0, 0, 0, 0];
                overlayIsBlack = 1;
            }
            do {
                let middle = (bottom + top) / 2;
                currentOverlay[3] = middle;
                currentColor = Util.applyAlpha(rgbBase, currentOverlay);
                let currentLuminance = convert.rgb.lch.raw(currentColor)[0];
                if (currentLuminance > desiredLuminance ? !overlayIsBlack : overlayIsBlack)
                    top = middle - 0.01;
                else
                    bottom = middle + 0.01;
            } while (!Util.checkLuminance(currentColor, desiredLuminance, threshold));
            currentOverlay[3] = Math.round(currentOverlay[3] * 100) / 100;
            exportedColors.push(new Color('rgba', [...currentOverlay]));
        });
        return exportedColors;
    }
}

export { Color, ConsistentShading };
